#include <iostream>
#include <cstdio>
#include <usb.h>
#include "usb_drv.h"
using namespace std;

unsigned int timeout = 2000;
unsigned int status;

struct usb_device *usb_find_device(void)
{
  struct usb_bus *bus;
  struct usb_device *dev;
  usb_init();
  usb_find_busses();
  usb_find_devices();
  for (bus = usb_busses; bus; bus = bus->next)
  {
    for (dev = bus->devices; dev; dev = dev->next)
    {
      if ((dev->descriptor.idVendor == MyUSB_VendorID) && (dev->descriptor.idProduct == MyUSB_ProductID))
        return dev;
    }
  }
  return 0;
}

struct usb_dev_handle *usb_claim_device(void)
{
  struct usb_device *dev;
  struct usb_dev_handle *handle;
  int set_conf_error, claim_int_error;
  dev = usb_find_device();
  if (dev == NULL) cout << "USB device not found!";
  handle = usb_open(dev);
  if (handle == NULL) cout << "USB device handle not found!";
  set_conf_error = usb_set_configuration(handle, 1);
  if ( set_conf_error < 0)
  {
    cout << "Can not set configuration (errorcode = " << set_conf_error << "!)";
    usb_close(handle);
  }
  claim_int_error = usb_claim_interface(handle, 0);
  if ( claim_int_error < 0)
  {
    cout << "Can not claim interface (errorcode = " << claim_int_error << "!)";
    usb_close(handle);
  }
  return handle;
}

void send_to_port(unsigned char portb)
{
struct usb_dev_handle *handle;
handle = usb_claim_device();
if (usb_bulk_write(handle, 0x01, (char*) &portb, 1, timeout) < 1)
   cout << "Can not set digital port B!";
usb_release_interface(handle, 0);
usb_close(handle);
}


unsigned int get_once_from_adc()
{
struct usb_dev_handle *handle;
unsigned char buf[2];
unsigned int adcvalue;
handle = usb_claim_device();

if (usb_bulk_read(handle, 0x82, (char*) buf, 2, timeout) < (2))
   cout << "USB_bulk_read error!"; // should never occur
adcvalue = (unsigned int) (buf[1]*256)+buf[0];

usb_release_interface(handle, 0);
usb_close(handle);
return adcvalue;
}

void get_from_adc(unsigned int* wordbuff2, unsigned int samples)
{
struct usb_dev_handle *handle;
unsigned char charbuff[EP2_FIFO_Size];
unsigned char *charbuffp;
handle = usb_claim_device();
unsigned int words_to_read;
int offset= -1;

// 6 = ADC1ms
if (usb_control_msg(handle, USB_VendorRequestCode, UC_ADC_Read, 6, (int) samples, NULL, 0, timeout) < 0)
    cout <<"USB_control_msg error!"; // should never occur

  while (samples)
  {
    offset++;
    if (samples > (EP2_FIFO_Size/2))
      words_to_read = (EP2_FIFO_Size/2);
    else
      words_to_read = samples;
    if (usb_bulk_read(handle, 0x82, (char*) charbuff, words_to_read * 2, timeout) < (words_to_read * 2))
      cout << "USB_bulk_read error!"; // should never occur
    charbuffp = charbuff;
    samples -= words_to_read;
    for (unsigned int i= 0; i < words_to_read; i++)
    {
      unsigned int sword;
      sword = (unsigned int) *charbuffp++;
      sword += (unsigned int) *charbuffp++ * 256;
      wordbuff2[offset+i] = sword;
    }
  }
