/* Copyright (c) 2009   Guy WEILER         weigu[at]weigu[dot]lu

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Licence: GPL-3        */


// http://thingylab.com/wiki/c_example_with_libusb-1.0
//gcc -o usb-test usb-test.c -I/usr/local -L/usr/local -lusb-1.0
#include <iostream>
#include <cstdio>
#include <libusb-1.0/libusb.h>
#include "usb_drv.h"
using namespace std;

unsigned int timeout = 500;
unsigned int status;



void send_to_port(unsigned char Port)
{
  libusb_device **list;
  libusb_device *found = NULL;
  libusb_context *ctx = NULL;
  libusb_device_handle *handle;
  struct libusb_device_descriptor desc;
  int err = 0, attached = 0, BytesWritten = 0;
  ssize_t i = 0;

  //Discover devices
  libusb_init(&ctx);
  libusb_set_debug(ctx,3);
  ssize_t cnt = libusb_get_device_list(ctx, &list);
  if (cnt < 0) cout << "no usb devices found\n";
  //Find the device
  for(i = 0; i < cnt; i++) {
    libusb_device *device = list[i];
    libusb_get_device_descriptor(device,&desc);
    if( desc.idVendor == VID && desc.idProduct == PID ) {
      found = device;
      break;
    }
  }
  //Get the handle
  if(found){
    //cout << "found usb-dev-block!\n";
    err = libusb_open(found, &handle);
    if (err) cout << "Unable to open usb device\n";
    if (libusb_kernel_driver_active(handle,0)){
      cout << "Device busy...detaching...\n";
      libusb_detach_kernel_driver(handle,0);
      attached = 1;
    }
    else //cout << "Device free from kernel\n";
    err = libusb_claim_interface( handle, 0 );
    if (err){
      cout << "Failed to claim interface. ";
      switch( err ){
        case LIBUSB_ERROR_NOT_FOUND: cout << "not found\n";	break;
        case LIBUSB_ERROR_BUSY:		 cout << "busy\n";		break;
        case LIBUSB_ERROR_NO_DEVICE: cout << "no device\n";	break;
        default:                     cout << "other\n";     break;
      }
    }
    if (libusb_bulk_transfer(handle, 0x02, (unsigned char*) &Port, 1, &BytesWritten, timeout))
      cout << "Fehler beim Schreiben aufgetreten!\n";
    int SByte = Port;
    cout << "wrote " << BytesWritten << " Bytes to to EP2. Byte = " << hex << SByte << dec << endl;
    //if we detached kernel driver, reattach.
    if (attached == 1) libusb_attach_kernel_driver(handle, 0);
    libusb_close(handle);
  }
  libusb_free_device_list(list, 1);
  libusb_exit(ctx);
}


unsigned int get_once_from_adc()
{
  libusb_device **list;
  libusb_device *found = NULL;
  libusb_context *ctx = NULL;
  libusb_device_handle *handle;
  struct libusb_device_descriptor desc;
  int err = 0, attached = 0, BytesWritten = 0;
  ssize_t i = 0;
  unsigned char buf[64];
  unsigned int adcvalue = 0;

  //Discover devices
  libusb_init(&ctx);
  libusb_set_debug(ctx,3);
  ssize_t cnt = libusb_get_device_list(ctx, &list);
  if (cnt < 0) cout << "no usb devices found\n";
  //Find the device
  for(i = 0; i < cnt; i++) {
    libusb_device *device = list[i];
    libusb_get_device_descriptor(device,&desc);
    if( desc.idVendor == VID && desc.idProduct == PID ) {
      found = device;
      break;
    }
  }
  //Get the handle
  if(found){
    //cout << "found usb-dev-block!\n";
    err = libusb_open(found, &handle);
    if (err) cout << "Unable to open usb device\n";
    if (libusb_kernel_driver_active(handle,0)){
      cout << "Device busy...detaching...\n";
      libusb_detach_kernel_driver(handle,0);
      attached = 1;
    }
    else //cout << "Device free from kernel\n";
    err = libusb_claim_interface( handle, 0 );
    if (err){
      cout << "Failed to claim interface. ";
      switch( err ){
        case LIBUSB_ERROR_NOT_FOUND: cout << "not found\n";	break;
        case LIBUSB_ERROR_BUSY:		 cout << "busy\n";		break;
        case LIBUSB_ERROR_NO_DEVICE: cout << "no device\n";	break;
        default:                     cout << "other\n";     break;
      }
    }
    if (libusb_bulk_transfer(handle, 0x81, (unsigned char*) buf, 64, &BytesWritten, timeout))
      cout << "Fehler beim Lesen aufgetreten!\n";
    adcvalue = (unsigned int) (buf[1]*256)+buf[0];
    cout << "got " << BytesWritten << " Bytes from EP1. ADCValue = " << adcvalue << endl;
    //if we detached kernel driver, reattach.
    if (attached == 1) libusb_attach_kernel_driver(handle, 0);
    libusb_close(handle);
  }
  libusb_free_device_list(list, 1);
  libusb_exit(ctx);
  return adcvalue;
}

